package com.flyer.foster.controller.app;

import com.baomidou.mybatisplus.extension.conditions.query.LambdaQueryChainWrapper;
import com.flyer.foster.dto.TreeAddDTO;
import com.flyer.foster.dto.TreeRespDTO;
import com.flyer.foster.dto.TreeUpdateDTO;
import com.flyer.foster.entity.FosterRecord;
import com.flyer.foster.entity.Tree;
import com.flyer.foster.pojo.StpAppUtil;
import com.flyer.foster.service.IFosterRecordService;
import com.flyer.foster.service.ITreeService;
import com.flyer.util.R;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.time.LocalDateTime;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 树（小程序）
 *
 * @author flyer
 * @since 2024-05-09
 */
@RestController
@RequestMapping("/app/tree")
public class AppTreeController {
    @Autowired
    private ITreeService iTreeService;

    /**
     * 根据果园id查询树
     *
     * @param gardenId
     * @return
     */
    @GetMapping("/list/{gardenId}")
    public R getTreeByGardenId(@PathVariable Integer gardenId) {
        return R.ok().result(iTreeService.getTreeById(gardenId));
    }

    /**
     * 修改树
     */
    @PutMapping("")
    public R update(@RequestBody TreeUpdateDTO updateDTO) {
        return R.ok().result(
                iTreeService.lambdaUpdate()
                        .set(Tree::getName, updateDTO.getGardenId())
                        .eq(Tree::getId, updateDTO.getId())
                        .update());
    }

    /**
     * get tree
     */
    @GetMapping("/{treeId}")
    public R get(@PathVariable Integer treeId) {
        Tree tree = iTreeService.lambdaQuery().eq(Tree::getName ,treeId).one();
        TreeRespDTO dto = new TreeRespDTO();
        BeanUtils.copyProperties(tree, dto);
        return R.ok().result(dto);
    }

    /**
     * 返回区域随机的一棵树
     *
     * @return
     */
    @GetMapping("/getRandomTree/{gardenId}")
    public R getRandomTree(@PathVariable Integer gardenId) {
        int tenantId = StpAppUtil.getTokenSession().getInt("tenantId");
        Tree tree = new Tree();
        tree.setGardenId(gardenId);
        tree.setTenantId(tenantId);
        tree.setCreatedBy("kelei");
        tree.setCreatedTime(LocalDateTime.now());
        tree.setUpdatedBy("kelei");
        tree.setUpdatedTime(LocalDateTime.now());
        // 根据果园id创建一颗树
        iTreeService.save(tree);
        // 获取新插入的树的信息并返回给前端
        return R.ok().result(iTreeService.getById(tree.getId()));
    }
}
